/*
 * Copyright (c) 2016 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "dotnet/dotnet_launcher.h"
#include "dotnet_signal_handler.h"
#include "utils.h"
#include "log.h"

#include <cstdio>
#include <vector>
#include <memory>

#include <Ecore.h>
#include <Eina.h>
#include <aul.h>

#define __XSTR(x) #x
#define __STR(x) __XSTR(x)

#ifndef VERSION
#define LAUNCHER_VERSION_STR "-Unknown-"
#else
#define LAUNCHER_VERSION_STR __STR(VERSION)
#endif

static std::string VersionOption("--version");
static std::string StandaloneOption("--standalone");

int main(int argc, char *argv[])
{
	registerSignalHandler();

	int i;
	bool standalone = false;
	const char* standalonePath = nullptr;

	std::vector<char*> vargs;

	// start index 1 to avoid passing executable name "dotnet-launcher" as a parameter
	for (i = 1;	i <argc; i++) {
		if (VersionOption.compare(argv[i]) == 0) {
			printf("Dotnet launcher Version %s\n", LAUNCHER_VERSION_STR);
			return 0;
		} else if (StandaloneOption.compare(argv[i]) == 0) {
			standalone = true;

			if (i > argc-1) {
				fprintf(stderr, "Assembly path must be after \"--standalone\" option\n");
				return 1;
			}
			i++;
			standalonePath = argv[i];
		} else {
			vargs.push_back(argv[i]);
		}
	}

	using tizen::runtime::LauncherInterface;
	using tizen::runtime::Launchpad;
	using tizen::runtime::AppInfo;
	std::unique_ptr<LauncherInterface> runtime;

	using tizen::runtime::dotnetcore::CoreRuntime;
	std::unique_ptr<LauncherInterface> coreRuntime(new CoreRuntime());
	runtime = std::move(coreRuntime);

	if (standalone) {
		_DBG("##### Run it standalone #########");
		const char* appId = getenv("AUL_APPID");
		_DBG("AUL_APPID : %s", appId);
		std::string appRoot;
		if (appId != nullptr) {
			const char* appRootPath = aul_get_app_root_path();
			if (appRootPath != nullptr)
				appRoot = std::string(appRootPath);
		} else {
			appId = "dotnet-launcher";
		}

		if (appRoot.empty())
			appRoot = baseName(baseName(standalonePath));
		if (runtime->initialize(true) != 0) {
			_ERR("Failed to initialize");
			return 1;
		}

		int argsLen = vargs.size();
		char** args = &vargs[0];
		if (runtime->launch(appId, appRoot.c_str(), standalonePath, argsLen, args)) {
			_ERR("Failed to launch");
			return 0;
		}
	} else {
		Launchpad.onCreate = [&runtime]() {
			if (runtime->initialize(false) != 0)
				_ERR("Failed to initialized");
			else
				_DBG("Success to initialized");
		};

		Launchpad.onTerminate = [&runtime](const AppInfo& appInfo, int argc, char** argv) {
			_DBG("terminated with app path : %s", appInfo.path.c_str());
			_DBG("appId : %s", appInfo.id.c_str());
			_DBG("pkg : %s", appInfo.pkg.c_str());
			_DBG("type : %s", appInfo.type.c_str());

			// The launchpad pass the name of exe file to the first argument.
			// For the C# spec, we have to skip this first argument.

			if (runtime->launch(appInfo.id.c_str(), appInfo.root.c_str(), appInfo.path.c_str(), argc-1, argv+1))
				_ERR("Failed to launch");
		};
		Launchpad.loaderMain(argc, argv);
	}

	return 0;
}
